@@ -1,7 +1,7 @@
 # ABSTRACT: Simpler REST client for Icinga2
 
 package Monitoring::Icinga2::Client::Simple;
-$Monitoring::Icinga2::Client::Simple::VERSION = '0.001000';
+$Monitoring::Icinga2::Client::Simple::VERSION = '0.002000';
 use strict;
 use warnings;
 use 5.010_001;
@@ -51,7 +51,7 @@ sub schedule_downtime {
 
 sub _schedule_downtime_type {
     my ($self, $type, $args) = @_;
-    my $req_results = $self->_verbose_request('POST',
+    my $req_results = $self->_request('POST',
         '/actions/schedule-downtime',
         {
             type => $type,
@@ -93,7 +93,7 @@ sub _remove_downtime_type {
     } else {
         @post_args = ( $args, { type => $type } );
     }
-    my $req_results = $self->_verbose_request('POST',
+    my $req_results = $self->_request('POST',
         "/actions/remove-downtime",
         @post_args,
     );
@@ -107,7 +107,7 @@ sub send_custom_notification {
 
     my $obj_type = defined $args{host} ? 'host' : 'service';
 
-    return $self->_verbose_request('POST',
+    return $self->_request('POST',
         '/actions/send-custom-notification',
         {
             type => ucfirst $obj_type,
@@ -126,7 +126,7 @@ sub set_notifications {
     _checkargs_any(\%args, qw/ host service /);
     my $uri_object = $args{service} ? 'services' : 'hosts';
 
-    return $self->_verbose_request('POST',
+    return $self->_request('POST',
         "/objects/$uri_object",
         {
             attrs => { enable_notifications => !!$args{state} },
@@ -138,7 +138,7 @@ sub set_notifications {
 sub query_app_attrs {
     my ($self) = @_;
 
-    my $r = $self->_verbose_request('GET',
+    my $r = $self->_request('GET',
         "/status/IcingaApplication",
     );
     # uncoverable branch true
@@ -162,14 +162,14 @@ sub query_app_attrs {
     sub set_app_attrs {
         my ($self, %args) = @_;
         _checkargs_any(\%args, keys %legal_attrs);
-        my @unknown_attrs = grep { not exists $legal_attrs{$_} } sort keys %args;
+        my @unknown_attrs = grep { not exists $legal_attrs{$_} } keys %args;
         @unknown_attrs and croak(
             sprintf "Unknown attributes: %s; legal attributes are: %s",
-            join(",", @unknown_attrs),
+            join(",", sort @unknown_attrs),
             join(",", sort keys %legal_attrs),
         );
 
-        return $self->_verbose_request('POST',
+        return $self->_request('POST',
             '/objects/icingaapplications/app',
             {
                 attrs => {
@@ -185,34 +185,54 @@ sub set_global_notifications {
     $self->set_app_attrs( notifications => $state );
 }
 
+sub query_hosts {
+    my ($self, %args) = @_;
+    _checkargs(\%args, qw/ hosts /);
+    my $result = $self->_request('GET',
+        '/objects/hosts',
+        { filter => _filter_expr( "host.name", $args{hosts} ) },
+    );
+}
+
 sub query_host {
     my ($self, %args) = @_;
     _checkargs(\%args, qw/ host /);
-    return $self->_verbose_request('GET',
-        '/objects/hosts',
-        { filter => "host.name==\"$args{host}\"" }
-    )->[0];
+    return $self->query_hosts( hosts => $args{host} )->[0];
 }
 
 sub query_child_hosts {
     my ($self, %args) = @_;
     _checkargs(\%args, qw/ host /);
-    return $self->_verbose_request('GET',
+    return $self->_request('GET',
         '/objects/hosts',
         { filter => "\"$args{host}\" in host.vars.parents" }
     );
 }
 
+sub query_parent_hosts {
+    my ($self, %args) = @_;
+    my $expand = delete $args{expand};
+    # uncoverable condition right
+    my $results = $self->query_host( %args ) // {};
+    # uncoverable condition right
+    my $names = $results->{attrs}{vars}{parents} // [];
+    undef $results;
+    # uncoverable condition right
+    return $names unless $expand and @$names;
+    return $self->query_hosts( hosts => $names );
+}
+
 sub query_services {
     my ($self, %args) = @_;
-    _checkargs(\%args, qw/ service /);
-    return $self->_verbose_request('GET',
+    _checkargs_any(\%args, qw/ service services /);
+    my $srv = $args{service} // $args{services};
+    return $self->_request('GET',
         '/objects/services',
-        { filter => "service.name==\"$args{service}\"" }
+        { filter => _filter_expr( "service.name", $srv ) },
     );
 }
 
-sub _verbose_request {
+sub _request {
     my ($self, $method, $url, $getargs, $postdata) = @_;
 
     if(defined $getargs and ref $getargs) {
@@ -220,12 +240,9 @@ sub _verbose_request {
         $postdata = $getargs;
         undef $getargs;
     }
-    _debug("Query URL: $url", $getargs ? "Query args: $getargs" : ());
-    _debug_dump($postdata);
     # uncoverable branch true
     my $r = $self->do_request($method, $url, $getargs, $postdata)
         or die $self->request_status_line . "\n";
-    _debug_dump($r);
     return $r->{results};
 }
 
@@ -256,21 +273,20 @@ sub _checkargs_any {
 # Not a method!
 sub _create_filter {
     my $args = shift;
-    croak( "`host' argument missing" ) unless defined $args->{host};
-    my $filter = "host.name==\"$args->{host}\"";
-    $filter .= " && service.name==\"$args->{service}\"" if $args->{service};
-    return $filter;
+    defined $args->{host} or croak(
+        sprintf( "missing or undefined argument `host' to %s()", (caller(1))[3] )
+    );
+    my $filter = _filter_expr( "host.name", $args->{host} );
+    return $filter unless $args->{service};
+    return "$filter && " . _filter_expr( "service.name", $args->{service} );
 }
 
-sub _debug {
-    print STDERR @_ if DEBUG;
-}
-
-sub _debug_dump {
-    if(DEBUG) {
-        require YAML::XS;
-        print YAML::XS::Dump(\@_);
-    }
+# Return an == or `in' expression depending on the type of argument.
+# Only scalars and arrayrefs make sense!
+sub _filter_expr {
+    my ($what, $arg) = @_;
+    return "$what==\"$arg\"" unless ref $arg;
+    return "$what in [" . join( ',', map { "\"$_\"" } @$arg ) . ']';
 }
 
 1;
@@ -287,7 +303,7 @@ Monitoring::Icinga2::Client::Simple - Simpler REST client for Icinga2
 
 =head1 VERSION
 
-version 0.001000
+version 0.002000
 
 =head1 SYNOPSIS
 
@@ -495,12 +511,24 @@ argument to also delete all of this host's service downtimes.
     $result = $ia->query_host( host => 'web-1' );
     say "$result->{attrs}{name}: $result->{attrs}{type}";
 
-Query all information Icinga2 has on a certain host. The result is a hashref,
-currently containing a single key C<attrs>. If the host is not found, C<undef>
-is returned.
+Provided for backwards compatibility only. Equivalent to L</query_hosts> except
+that it accepts only a single host name and the result is a reference to a
+single hash, or C<undef> if the host can't be found.
 
 The only mandatory argument is C<host>.
 
+=head2 query_hosts
+
+    $results = $ia->query_host( hosts => [ qw( web-1 hypervisor-1 ) ] );
+    say "$_->{attrs}{name}: $_->{attrs}{type}" for @$results;
+
+Query all information Icinga2 has on a number of hosts. The result is a
+reference to a list of hashes, each of which, as per the current API version,
+contains a single key named C<attrs>.
+
+The only mandatory argument is C<hosts>, which may also specify a single
+scalar, so this method is preferable over L</query_host>.
+
 =head2 query_child_hosts
 
     $results = $ia->query_child_hosts( host => 'hypervisor-1' );
@@ -511,18 +539,28 @@ is a reference to a list of hashes like those returned by L</query_host>.
 
 The only mandatory argument is C<host>.
 
+=head2 query_parent_hosts
+
+    $results = $ia->query_parent_hosts( host => 'web-1', expand => 1 );
+    say for @$results;
+
+Query all parent host names of a given host. If C<expand> is C<false> or
+missing, the result is a reference to a list of names. Otherwise it is a
+reference to a list of hashes like those returned by L</query_host>.
+
+The only mandatory argument is C<host>.
+
 =head2 query_services
 
     $result = $ia->query_services( service => 'HTTP' );
+    $result = $ia->query_services( services => [ qw( HTTP SMTP ) ] );
     say "$_->{attrs}{name}: $_->{attrs}{type}" for @$results;
 
-Query all information Icinga2 has on a certain service. As services usually
-have more than one instance, the result is a reference to a list of hashes,
-each describing one instance.
+Query all information Icinga2 has on a number of services. As services usually
+have more than one instance, the result is always a reference to a list of hashes,
+each describing one instance, even when querying a single service.
 
-The only mandatory argument is C<service>. Note that this is a singular as it
-specifies a single service name while the method name is plural due to the
-plurality of returned results.
+The only mandatory argument is C<service> I<or> C<services>.
 
 =head2 send_custom_notification
 
